import { Component, Input, ViewChild, ChangeDetectionStrategy, ContentChildren, HostListener, NgModule } from '@angular/core';

class SOptionComponent {
}
SOptionComponent.decorators = [
    { type: Component, args: [{
                selector: 'abs-option',
                template: `
		<div #content>
			<ng-content></ng-content>
		</div>
	`
            },] }
];
SOptionComponent.propDecorators = {
    title: [{ type: Input }],
    value: [{ type: Input }],
    selected: [{ type: Input }],
    disabled: [{ type: Input }],
    isInvalid: [{ type: Input }],
    content: [{ type: ViewChild, args: ['content',] }]
};
class AngularBootstrapSelectComponent {
    constructor() {
        this.insideClick = () => this.clickInside = true;
        this.shown = [];
        this.open = false;
        this.focusIndex = -1;
        this.clickInside = false;
    }
    onKeyDown(evt) {
        switch (evt.key) {
            case "Enter":
                {
                    if (this.focusIndex >= 0 && !this.shown[this.focusIndex].disabled)
                        this.optionSelect(this.shown[this.focusIndex]);
                    break;
                }
            case 'ArrowDown':
                {
                    if (this.focusIndex == this.shown.length - 1)
                        this.focusIndex = 0;
                    else
                        this.focusIndex += 1;
                    break;
                }
            case 'ArrowUp':
                {
                    if (this.focusIndex == 0)
                        this.focusIndex = this.shown.length - 1;
                    else
                        this.focusIndex -= 1;
                    break;
                }
        }
    }
    outsideClick() {
        if (!this.clickInside)
            this.open = false;
        this.clickInside = false;
    }
    // riconosce se c'è un'opzione che ha per TITOLO ESATTO ciò che viene scritto nella input
    // il titolo viene comparato senza spazi iniziali/finali (trim) e in minuscolo (toLowerCase)
    recognizeInput(e) {
        let corrOption, input, value;
        input = e.target;
        value = input.value.toLowerCase().trim();
        corrOption = this.children.find(e => e.title.toLowerCase().trim() === value);
        if (!corrOption || corrOption.disabled) {
            let selected = this.children.filter(op => !!op.selected)[0];
            input.value = selected ? selected.title : '';
        }
        else {
            this.optionSelect(corrOption);
            input.value = corrOption.title;
        }
    }
    getSelected() {
        return this.children.filter(op => !!op.selected);
    }
    onControlClick() {
        if (!this.open)
            this.open = true;
        if (this.multiple)
            this.input.nativeElement.focus();
    }
    search(evt) {
        this.focusIndex = -1;
        let key = evt.target.value.trim().toLowerCase();
        this.shown = this.children.filter(op => op.title.toLowerCase().includes(key) ||
            op.value.toLowerCase().includes(key));
    }
    adjustPosition() {
        //console.log( this.list.nativeElement.offsetTop, this.list.nativeElement.offsetHeight );
        this.list.nativeElement.style.top = `${this.control.nativeElement.offsetHeight}px`;
    }
    setupOptions() {
        this.shown = [...this.children.toArray()];
    }
    getControlText() {
        if (!this.children)
            return '';
        let selected = this.children.filter(op => !!op.selected);
        if (selected.length > 0)
            return selected[0].title;
        else
            return '';
    }
    optionSelect(option) {
        if (!this.multiple)
            this.children.forEach(op => op.selected = false);
        this.children.find(el => el.value == option.value).selected = true;
        if (this.multiple) {
            this.input.nativeElement.value = '';
        }
        if (!this.multiple)
            this.open = false;
        if (!!this.onChange)
            this.onChange(this.children.map(el => el.value));
    }
    deselect(op) {
        this.children.find(el => el.value == op.value).selected = false;
        if (!!this.onChange)
            this.onChange(this.children.map(el => el.value));
    }
    hasFocus(op) {
        if (!this.shown.find(el => el.value == op.value))
            return false;
        return this.children.toArray().indexOf(op) == this.focusIndex % this.shown.length;
    }
    hoverOption(op) {
        this.input.nativeElement.focus();
        this.focusIndex = this.shown.indexOf(op);
    }
    ngOnChanges(changes) {
        console.log(changes);
    }
    ngAfterViewInit() {
        this.setupOptions();
        this.children.changes.subscribe(() => this.setupOptions());
    }
    ngOnInit() { }
}
AngularBootstrapSelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'abs-select',
                template: "<div \n\tclass=\"abs-select\"\n\t[attr.multiple]=\"multiple\"\n\t[attr.active]=\"open\" \n\t[attr.invalid]=\"invalid\"\n>\n\t<input \n\t\t#input \n\t\t*ngIf=\"!multiple\" \n\t\tclass=\"abs-control form-control\" \n\t\t[readOnly]=\"!open\"\n\t\t[value]=\"getControlText()\"\n\t\t(input)=\"search( $event )\"\n\t\t(click)=\"onControlClick()\"\n\t\t(change)=\"recognizeInput( $event )\"\n\t/>\n\n\t<div \n\t\t#control\n\t\t*ngIf=\"multiple\"\n\t\tclass=\"abs-control form-control\"\n\t\t(click)=\"onControlClick()\"\n\t>\n\t\t<span \n\t\t\tclass=\"abs-pill\"\n\t\t\t*ngFor=\"let p of getSelected()\"\n\t\t>\n\t\t\t{{p.title}}\n\t\t\t<i (click)=\"deselect( p )\">&times;</i>\n\t\t</span>\n\n\t\t<div class=\"clearfix\"></div>\n\t\t\n\t\t<input \n\t\t\t#input\n\t\t\t[readOnly]=\"!open\"\n\t\t\t(input)=\"search( $event )\"\n\t\t\t(change)=\"recognizeInput( $event )\"\n\t\t/>\n\t</div>\n\n\t<ul #list\n\t\tclass=\"list-group abs-list mt-2\">\n\t\t<li \n\t\t\t*ngFor=\"let el of shown\"\n\t\t\tclass=\"list-group-item\"\n\t\t\t[attr.active]=\"!!el.selected\"\n\t\t\t[attr.disabled]=\"!!el.disabled\"\n\t\t\t[attr.value]=\"el.value\"\n\t\t\t[attr.focus]=\"this.hasFocus( el )\"\n\t\t\t(click)=\"optionSelect(el)\"\n\t\t\t(mouseover)=\"hoverOption(el)\"\n\t\t>\n\t\t\t{{el.title}}\n\t\t\t<div \n\t\t\t\tclass=\"mt-3\"\n\t\t\t\t*ngIf=\"el.content.nativeElement.innerText.length > 0\" \n\t\t\t\t[innerHTML]=\"el.content.nativeElement.innerHTML\"\n\t\t\t>\n\t\t\t</div>\n\t\t</li>\n\t</ul>\n</div>",
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [".clearfix{clear:both}.abs-select input:not(.form-control){-moz-appearance:none;-webkit-appearance:none;appearance:none;border:none}.abs-select input:not(.form-control):focus{outline:none}.abs-select .form-control:focus{border-color:#ced4da;box-shadow:none}.abs-select .form-control[readonly]{background-color:#fff;color:#495057}.abs-select[invalid=true] .abs-control{border-color:#dc3545}.abs-select[invalid=true][active=true] .abs-control{background-color:#fff;border-color:#efa2a9!important;box-shadow:0 0 0 0 0 0 .2rem rgba(220,53,69,.25);color:#495057;outline:0}.abs-select[active=true] .abs-control{background-color:#fff;border-color:#80bdff!important;box-shadow:0 0 0 .2rem rgba(0,123,255,.25);color:#495057;outline:0}.abs-select[active=true] .abs-list{display:flex}.abs-select[multiple=true] .abs-control{border:1px solid #ced4da;border-radius:.25rem;height:unset;max-height:unset;min-height:calc(1.5em + .75rem + 2px)}.abs-select[multiple=true] .abs-control>input{display:block;height:calc(1.5em + 2px);width:100%}.abs-select[multiple=true] .abs-control>.abs-pill{align-items:center;background-color:#f3f4f5;border:1px solid #dee2e6;border-radius:.25rem;display:flex;float:left;font-size:small;font-weight:400;margin:0 .25rem .375rem 0;padding:.25rem 1rem;width:auto}.abs-select[multiple=true] .abs-control>.abs-pill>i{cursor:pointer;height:100%;padding-left:.5rem}.abs-select .abs-list{display:none;width:100%}.abs-select .abs-list>li{cursor:pointer;position:static}.abs-select .abs-list>li[focus=true]{background-color:#f4f5f7}.abs-select .abs-list>li[active=true]{background-color:#007bff;border-color:#007bff;color:#fff}.abs-select .abs-list>li[disabled=false][active=true]:hover,.abs-select .abs-list>li[disabled=false][active=true][focus=true]{background-color:#0069d9;border-color:#007bff;color:#fff}.abs-select .abs-list>li[disabled=true]{background-color:#fff;color:#6c757d;cursor:not-allowed!important;pointer-events:none}"]
            },] }
];
AngularBootstrapSelectComponent.ctorParameters = () => [];
AngularBootstrapSelectComponent.propDecorators = {
    invalid: [{ type: Input }],
    multiple: [{ type: Input }],
    required: [{ type: Input }],
    onChange: [{ type: Input }],
    input: [{ type: ViewChild, args: ['input',] }],
    control: [{ type: ViewChild, args: ['control',] }],
    list: [{ type: ViewChild, args: ['list',] }],
    children: [{ type: ContentChildren, args: [SOptionComponent,] }],
    insideClick: [{ type: HostListener, args: ['click',] }],
    onKeyDown: [{ type: HostListener, args: ['keydown', ['$event'],] }],
    outsideClick: [{ type: HostListener, args: ['document:click',] }]
};

class AngularBootstrapSelectModule {
}
AngularBootstrapSelectModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AngularBootstrapSelectComponent],
                imports: [],
                exports: [AngularBootstrapSelectComponent]
            },] }
];

/*
 * Public API Surface of angular-bootstrap-select
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AngularBootstrapSelectComponent, AngularBootstrapSelectModule, SOptionComponent };
//# sourceMappingURL=angular-bootstrap-select.js.map
